@@ -20,16 +20,15 @@ from csv import QUOTE_NONNUMERIC  # 2 # noqa
 from csv import QUOTE_NONE        # 3 # noqa
 import dask.dataframe as dd
 import datatable as dt
-from files import vec_to_hdf5
-import h5py
-from nessvec.util import cosine_similarity
 import numpy as np
 import pandas as pd
 from pathlib import Path
-# import pynndescent as pynn
 import pyarrow as pa
 import pyarrow.parquet as pq
 import sys
+
+from nessvec.files import vec_to_hdf5, load_glove_dask
+from nessvec.util import cosine_similarity
 # from urllib.request import urlretrieve
 
 
@@ -38,27 +37,6 @@ STANFORD_GLOVE_PATH = DATA_DIR / 'glove.6B.300d.txt'
 WIKINEWS_FASTTEXT_PATH = DATA_DIR / 'wiki-news-300d-1M.vec'
 
 
-def load_glove(subword=False, filepath=STANFORD_GLOVE_PATH,
-               skiprows=0, quoting=QUOTE_NONE, sep=' ', header=None,
-               dtype={0: str}, **kwargs):
-    """ Load Stanford GloVe vectors into dask.dataframe """
-    filepath = str(filepath)
-    ddf = dd.read_csv(
-        filepath,
-        header=header,
-        skiprows=skiprows,
-        dtype=dtype,
-        quoting=quoting,
-        sep=sep)
-    # set_index doesn't work in a dask.dataframe
-    # and parquet can't store strings in index
-    tokens = np.array([str(x) for x in ddf[0]])
-    vocab = dict(zip(tokens, range(len(tokens))))
-    ddf = ddf[ddf.columns[1:]]
-    ddf.columns = [str(c) for c in ddf.columns]
-    return ddf, vocab
-
-
 def load_fasttext_vec(subword=False, filepath=WIKINEWS_FASTTEXT_PATH,
                       skiprows=1, quoting=QUOTE_NONE, sep=' ', header=None, dtype={0: str}, **kwargs):
     """ Load fasttext vectors from local file into dask.dataframe
@@ -143,33 +121,6 @@ def create_str_dataset(h5py_file, data, name='strings'):
     return dset
 
 
-def load_hdf5(filepath=str(WIKINEWS_FASTTEXT_PATH) + '.hdf5', encoding='utf8'):
-    hdf5_file = h5py.File(filepath, 'r')
-    vecs = hdf5_file['vecs']
-    vocab = hdf5_file['vocab']
-    vocab = pd.Series(data=range(len(vecs)), index=(s.decode(encoding) for s in vocab))
-    if len(vocab) != len(vecs):
-        print(f'vocab len = {len(vocab)} but vecs len = {len(vecs)}')
-    # for s in hdf5_file['vocab']:
-    #     try:
-    #         vocab += [s.decode('utf8')]
-    #     except UnicodeDecodeError:
-    #         vocab += [s]
-    #         print(vocab[max(-5, -len(vocab)):])
-    return vecs, vocab
-
-    # if not filepath_hdf.is_file():
-    #     with h5py.File(str(filepath_hdf), 'w') as f:
-    #         f.create_dataset(name, data=ddf.values)
-    #         f.create_dataset(f'{name}_index', data=np.arange(num_vecs))
-    #         # f.create_dataset(f'{name}_columns', data=np.array([f'x{i}' for i in np.arange(num_dims)]))
-    # # TODO: THIS WORKS FAST!!!
-    # h5file = h5py.File(str(filepath_hdf), 'r')
-    # # f = h5py.File('fasttext.hdf'), 'r')
-    # vecs.head()
-    # return vecs
-
-
 def read_parquet_dataframe(filepath=str(WIKINEWS_FASTTEXT_PATH) + '.parquet', kind='pandas'):
     """ Load a parquet file into a pandas or dask DataFrame
 
@@ -203,7 +154,9 @@ class H5DataFrame(pd.DataFrame):
 """
 
 
-class Index(pd.Series):
+class Loc(pd.Series):
+    """ Implements the Pandas .loc[obj] and .iloc[int] reverse index getters for any class """
+
     def __init__(self, data, index=None):
         if index is None:
             index = range(len(data))
@@ -225,8 +178,8 @@ class H5DataFrame():
             range(self.shape[1]), index=self.columns.values)
         self.reverse_index = pd.Series(
             range(self.shape[0]), index=self.index.values)
-        self.loc = Index(data=self.data, index=self.index)
-        self.iloc = Index(data=self.data)
+        self.loc = Loc(data=self.data, index=self.index)
+        self.iloc = Loc(data=self.data, index=range(len(self.data)))
 
     def __getitem__(self, label):
         return self.data[self.reverse_columns[label]]
@@ -415,7 +368,7 @@ def main():
     elif 'fasttext' in filepath.lower():
         ddf, vocab = load_fasttext_vec(filepath)
     elif 'glove' in filepath.lower():
-        ddf, vocab = load_glove(filepath)
+        ddf, vocab = load_glove_dask(filepath)
 
     num_vecs = ddf.shape[0].compute()
     num_dims = ddf.shape[1]  # noqa
