@@ -14,12 +14,16 @@ References:
 
 """
 import re
-
-import pandas as pd
-import numpy as np
 import logging
+from pathlib import Path
+import urllib
+
+import numpy as np
+from tqdm import tqdm
+
+from .constants import LARGE_FILES, DATA_DIR
+from .re_patterns import cre_url
 
-from nessvec.files import load_glove, DATA_DIR
 
 log = logging.getLogger(__name__)
 
@@ -114,45 +118,88 @@ def cosine_similarities(target, components):
     return components.dot(target).flatten()
 
 
-class IndexedVectors:
-    def __init__(self, vectors=None, index=None, normalizer=glove_normalize):
-        self.normalizer = normalizer
-        if vectors is None:
-            self.load()
-        elif isinstance(vectors, dict):
-            self.df = pd.DataFrame(vectors)
-        else:
-            self.df = pd.DataFrame(vectors, index=(index or range(len(vectors))))
+class DownloadProgressBar(tqdm):
+    """ Utility class that adds tqdm progress bar to urllib.request.urlretrieve
 
-    def load(self, dim=50, size=6):
-        self.df = pd.DataFrame(load_glove(dim=dim, size=size))
-        return self
+    >>> filemeta = LARGE_FILES['floyd']
+    >>> filename = Path(filemeta['path']).name
+    >>> url  = filemeta['url']
+    >>> dest_path  = str(Path(filemeta['path']))
+    >>> with DownloadProgressBar(unit='B', unit_scale=True, miniters=1, desc=filename) as dpb:
+    ...     urllib.request.urlretrieve(url, filename=dest_path, reporthook=dpb.update_to)
+    ('...nessvec/data/corpora/wikipedia/floyd.pkl', <http.client.HTTPMessage...>)
+    """
 
-    def get(self, key, default=None):
-        if key in self.df.columns:
-            return self.df[key]
-        return default
+    def update_to(self, b=1, bsize=1, tsize=None):
+        if tsize is not None:
+            self.total = tsize
+        self.update(b * bsize - self.n)
 
-    def __getitem__(self, key):
+
+def looks_like_url(url):
+    """ Simplified check to see if the text appears to be a URL.
+
+    Similar to `urlparse` but much more basic.
+
+    Returns:
+      True if the url str appears to be valid.
+      False otherwise.
+
+    >>> url = looks_like_url("totalgood.org")
+    >>> bool(url)
+    True
+    """
+    if not isinstance(url, str):
+        return False
+    if not isinstance(url, str) or len(url) >= 1024 or not cre_url.match(url):
+        return False
+    return True
+
+
+def download_if_necessary(
+        url_or_name,
+        dest_path=None):
+    """ Loads the file found in the local DATA_DIR or download it from the LARGE_FILES url
+
+    >>> download_if_necessary('')
+    """
+    if not url_or_name:
+        return None
+    file_meta = LARGE_FILES.get(url_or_name) or {'url': url_or_name}
+    log.debug(f'file_meta: {file_meta}')
+    url = file_meta['url']
+    if not dest_path and not looks_like_url(url):
+        if Path(url).is_file():
+            dest_path = url
+        if Path(DATA_DIR, url).is_file():
+            dest_path = url
+    dest_path = dest_path or file_meta.get('path')
+    # TODO: walk down from URL filename to match up with directories in DATA_DIR to build dest path
+    if not dest_path:
+        dest_path = file_meta.get('path', Path(DATA_DIR, file_meta.get('filename', Path(url).name)))
+    filename = Path(dest_path).name
+    if Path(dest_path).is_file():
+        with urllib.request.urlopen(url) as file:
+            cloud_file_size = file.info()['Content-Length']
+        local_file_size = str(dest_path.stat().st_size)
+        if cloud_file_size == local_file_size:
+            return dest_path
+    with DownloadProgressBar(unit='B', unit_scale=True, miniters=1, desc=filename) as dpb:
         try:
-            return self.df[key]
-        except KeyError:
-            print(f"Unable to find '{key}' in {self.df.shape} DataFrame of vectors")
-        normalized_key = self.normalizer(str(key))
-        try:
-            return self.df[normalized_key]
-        except KeyError:
-            print(f"Unable to find '{normalized_key}' in {self.df.shape} DataFrame of vectors")
-        raise(KeyError(f"Unable to find any of {set([key, normalized_key])} in self.df.shape DataFrame of vectors"))
+            urllib.request.urlretrieve(url, filename=dest_path, reporthook=dpb.update_to)
+        except (urllib.error.HTTPError, ValueError):
+            log.error(f'Unable to download file from "{url}" to "{dest_path}" using file_meta: {file_meta}.')
+            return None
+    return dest_path
 
-    def keys(self):
-        return self.df.columns.values
 
-    def values(self):
-        return self.df.T.values
-
-    def iteritems(self):
-        return self.df.T.iterrows()
-
-    def iterrows(self):
-        return self.df.T.iterrows()
+def filter_vocab(df, vocab):
+    # filter out the rows with these words
+    # ...['unfetter', 'unfrock', 'abash']
+    str_columns = [c for c in df.columns if hasattr(df[c], "str")]
+    analogy_words = np.unique(df[str_columns].values.reshape((-1,)))
+    analogy_words = np.intersect1d(analogy_words, vocab.index.unique())
+    is_valid = [df[c].isin(np.unique(vocab.index.values)) for c in str_columns]
+    is_valid = np.sum(is_valid, axis=0)
+    is_valid = is_valid == len(str_columns)
+    return df[is_valid].copy()
